#include <avr/io.h>
#include <avr/interrupt.h>
#include <LiquidCrystal.h>

LiquidCrystal lcd(11, 10, 6, 5, 4, 2);

uint16_t syncPhaseAcc;
uint16_t syncPhaseInc;
uint16_t grainPhaseAcc;
uint16_t grainPhaseInc;
uint16_t grainAmp;
uint8_t grainDecay;
uint16_t grain2PhaseAcc;
uint16_t grain2PhaseInc;
uint16_t grain2Amp;
uint8_t grain2Decay;

// Mappa dei canali analogici
#define GRAIN_FREQ_CONTROL  (0)
#define GRAIN2_DECAY_CONTROL (1)
#define GRAIN_DECAY_CONTROL  (2)
#define GRAIN2_FREQ_CONTROL  (3)
#define SYNC_CONTROL (4)

#define PWM_PIN       3
#define PWM_VALUE     OCR2B
#define LED_PIN       13
#define LED_PORT      PORTB
#define PWM_INTERRUPT TIMER2_OVF_vect

// Smooth logarithmic mapping
uint16_t antilogTable[] = {
64830,64132,63441,62757,62081,61413,60751,60097,59449,58809,58176,57549,56929,56316,55709,55109, 54515,53928,53347,52773,52204,51642,51085,50535,49991,49452,48920,48393,47871,47356,46846,46341, 45842,45348,44859,44376,43898,43425,42958,42495,42037,41584,41136,40693,40255,39821,39392,38968,  38548,38133,37722,37316,36914,36516,36123,35734,35349,34968,34591,34219,33850,33486,33125,32768
};
uint16_t mapPhaseInc(uint16_t input) {
  return (antilogTable[input & 0x3f]) >> (input >> 6);
}

// Scala cromatica
uint16_t midiTable[] = {
  17,18,19,20,22,23,24,26,27,29,31,32,34,36,38,41,43,46,48,51,54,58,61,65,69,73,
  77,82,86,92,97,103,109,115,122,129,137,145,154,163,173,183,194,206,218,231,
  244,259,274,291,308,326,346,366,388,411,435,461,489,518,549,581,616,652,691,
  732,776,822,871,923,978,1036,1097,1163,1232,1305,1383,1465,1552,1644,1742,
  1845,1955,2071,2195,2325,2463,2610,2765,2930,3104,3288,3484,3691,3910,4143,
  4389,4650,4927,5220,5530,5859,6207,6577,6968,7382,7821,8286,8779,9301,9854, 10440,11060,11718,12415,13153,13935,14764,15642,16572,17557,18601,19708,20879,
  22121,23436,24830,26306
};
uint16_t mapMidi(uint16_t input) {
  return (midiTable[(1023-input) >> 3]);
}

void audioOn() {
#if defined(__AVR_ATmega8__)
  TCCR2 = _BV(WGM20) | _BV(COM21) | _BV(CS20);
  TIMSK = _BV(TOIE2);
#elif defined(__AVR_ATmega1280__)
  TCCR3A = _BV(COM3C1) | _BV(WGM30);
  TCCR3B = _BV(CS30);
  TIMSK3 = _BV(TOIE3);
#else
  // Imposta il PWM a 31.25kHz, fase accurata
  TCCR2A = _BV(COM2B1) | _BV(WGM20);
  TCCR2B = _BV(CS20);
  TIMSK2 = _BV(TOIE2);
#endif
}

void setup() {
// Imposta l'audio
  pinMode(PWM_PIN,OUTPUT);
  audioOn();
  pinMode(LED_PIN,OUTPUT);

// Imposta il display
  lcd.begin(16, 2);
  lcd.setCursor(0,0);
  lcd.print("P1 P2 P3 P4 P5");
}

void loop() {
  // Mappatura della frequenza 
  //syncPhaseInc = mapPhaseInc(analogRead(SYNC_CONTROL)) / 4;
 
  
  // Mappatura di note MIDI: C, Db, D, Eb, E, F...
  syncPhaseInc = mapMidi(analogRead(SYNC_CONTROL));
  
// Mappatura di scala pentatonica: D, E, G, A, B 
// (si pu usare questa al posto della scala cromatica togliendo il commento)
// syncPhaseInc = mapPentatonic(analogRead(SYNC_CONTROL));

  grainPhaseInc  = mapPhaseInc(analogRead(GRAIN_FREQ_CONTROL)) / 2;
  grainDecay     = analogRead(GRAIN_DECAY_CONTROL) / 8;
  grain2PhaseInc = mapPhaseInc(analogRead(GRAIN2_FREQ_CONTROL)) / 2;
  grain2Decay    = analogRead(GRAIN2_DECAY_CONTROL) / 4;

// Funzioni per la visualizzazione deo parametri sul display
  lcd.setCursor(0,1);    
  lcd.print(analogRead(0)/110);
  lcd.setCursor(3,1);    
  lcd.print(analogRead(1)/110);
  lcd.setCursor(6,1);    
  lcd.print(analogRead(2)/110);
  lcd.setCursor(9,1);    
  lcd.print(analogRead(3)/110);
  lcd.setCursor(12,1);    
  lcd.print(analogRead(4)/110);

}

SIGNAL(PWM_INTERRUPT)
{
  uint8_t value;
  uint16_t output;

  syncPhaseAcc += syncPhaseInc;
  if (syncPhaseAcc < syncPhaseInc) {
    // Inizio del grano successivo
    grainPhaseAcc = 0;
    grainAmp = 0x7fff;
    grain2PhaseAcc = 0;
    grain2Amp = 0x7fff;
   // LED_PORT ^= 1 << LED_BIT; // Faster than using digitalWrite
  }
  
  // Incrementa la fase degli oscillatori granulari
  grainPhaseAcc += grainPhaseInc;
  grain2PhaseAcc += grain2PhaseInc;

  // Converte la fase in un'onda triangolare a 
  value = (grainPhaseAcc >> 7) & 0xff;
  if (grainPhaseAcc & 0x8000) value = ~value;
  // Moltiplica per l'ampiezza del grano corrente per ottenere il campione
  output = value * (grainAmp >> 8);

  // Ripete per il secondo grano
  value = (grain2PhaseAcc >> 7) & 0xff;
  if (grain2PhaseAcc & 0x8000) value = ~value;
  output += value * (grain2Amp >> 8);

 // Effettua il decadimento delle ampiezze del grano di un fattore ogni campione 
// (decadimento esponenziale)
  grainAmp -= (grainAmp >> 8) * grainDecay;
  grain2Amp -= (grain2Amp >> 8) * grain2Decay;

// Uscita della cala per la gamma disponibile, con taglio, se necessario
  output >>= 9;
  if (output > 255) output = 255;

// Uscita PWM (questo  pi veloce di analogWrite)  
  PWM_VALUE = output;
}






