/*
Pierduino 2012 - ROM Piano Player by Pier Calderan.
Collegare opzionalmente un display 16x2 rispettando la piedinatura come indicato.
Collegare opzionalmente un potenziometro da 4,7k alla porta analogica 5.
Uscita PWM impostata sul pin 3, opzionalmente sul pin 11 (se si cambia piedinatura del display).
Viene riprodotto un suono PCM di pianoforte a intervalli e frequenza regolati dal potenziometro, caricato con sounddata.h.
Essendo una lettura diretta del campione, a frequenza più alta corrisponde un tempo più breve e viceversa.
*/

#include "avr/pgmspace.h"
#include <LiquidCrystal.h>
LiquidCrystal lcd(11, 10, 6, 5, 4, 2); // attenzione alla piedinatura del display!
#include "sounddata.h"

#define SAMPLE_RATE 6000

volatile uint16_t sample;
byte lastSample;

// tabella di 256 valori seno nella memoria programma
unsigned int freqOut=8;

// variabili per l'interrupt dichiarate volatili
volatile byte IntCNT;              
volatile byte IntCNT1;             
volatile byte counter_ms;         //counter fittizio per poter disabiliatare il timer0 (quello usato dalla funzione delay())
volatile unsigned long Phase;     // accumulatore fase 32 bit
volatile unsigned long Freqword;  // parola 32 bit

void setup()
{
  lcd.begin(16, 2);
  lcd.setCursor(0,0);
  lcd.print("ROM Player");
  
  pinMode(3, OUTPUT);      // pin 3 = PWM out frequency: NOTA cambiare la corrispondente porta OCR2B e i bit COM2B0 e COM2B1 del timer2
//pinMode(11, OUTPUT);     // pin 11= PWM out frequency: NOTA cambiare la corrispondente porta OCR2A e i bit COM2A0 e COM2A1 del timer2

startPlayback();
}

void loop()
{
freqOut=(analogRead(5)/127)*400;           // decommentare per leggere il valore da 0 a 1023 dal potenziometro opzionale collegato alla porta analogica A5
delay(200);
    // visualizza la frequenza sul display a intervalli di un 1/5 secondo per evitare il flickering
    lcd.setCursor(0,1);
    lcd.print("Freq.  ");
    lcd.print(SAMPLE_RATE + freqOut);      
    
}



//******************************************************************
//******************************************************************
// Timer2 Interrupt a 31372,550 KHz = 32uSec (REFCLOCK per il generatore) runtime di 8 microsecondi
ISR(TIMER1_COMPA_vect) {
    if (sample >= sounddata_length) {
        if (sample == sounddata_length + lastSample) {
            stopPlayback();
        }
        else {
            OCR2B = sounddata_length + lastSample - sample;
        }
    }
    else {
        OCR2B = pgm_read_byte(&sounddata_data[sample]);
    }

    ++sample;
}

void startPlayback()
{


    ASSR &= ~(_BV(EXCLK) | _BV(AS2));

    TCCR2A |= _BV(WGM21) | _BV(WGM20);
    TCCR2B &= ~_BV(WGM22);

    TCCR2A = (TCCR2A | _BV(COM2B1)) & ~_BV(COM2B0);
    TCCR2B &= ~(_BV(COM2B1) | _BV(COM2B0));

    TCCR2B = (TCCR2B & ~(_BV(CS12) | _BV(CS11))) | _BV(CS10);

    OCR2B = pgm_read_byte(&sounddata_data[0]);
    cli();

    TCCR1B = (TCCR1B & ~_BV(WGM13)) | _BV(WGM12);
    TCCR1A = TCCR1A & ~(_BV(WGM11) | _BV(WGM10));
    TCCR1B = (TCCR1B & ~(_BV(CS12) | _BV(CS11))) | _BV(CS10);

    OCR1A = F_CPU / (SAMPLE_RATE + freqOut);    

    TIMSK1 |= _BV(OCIE1A);

    lastSample = pgm_read_byte(&sounddata_data[sounddata_length-1]);
    sample = 0;
    sei();
}

void stopPlayback()
{
    TIMSK1 &= ~_BV(OCIE1A);
    TCCR1B &= ~_BV(CS10);
    TCCR2A &= ~_BV(CS10);
    digitalWrite(3, LOW);
    startPlayback();
    delay(20);
}
