/*
Pierduino 2012 - Square Wave Generator by Pier Calderan.
Collegare opzionalmente un display 16x2 rispettando la piedinatura come indicato.
Collegare opzionalmente un potenziometro da 4,7k alla porta analogica 5.
Uscita PWM impostata sul pin 3, opzionalmente sul pin 11 (se si cambia piedinatura del display).
*/

#include "avr/pgmspace.h"
#include <LiquidCrystal.h>
LiquidCrystal lcd(11, 10, 6, 5, 4, 2); // attenzione alla piedinatura del display!

unsigned char RAMTable[256]; // tabella valori per la creazione di qualsiasi forma d'onda

double freqOut;
const double REFCLOCK=31376.6;      // clock di riferimento

// variabili per l'interrupt dichiarate volatili
volatile byte IntCNT;              
volatile byte IntCNT1;             
volatile byte counter_ms;         //counter fittizio per poter disabiliatare il timer0 (quello usato dalla funzione delay())
volatile unsigned long Phase;     // accumulatore della fase 32 bit
volatile unsigned long Freqword;  // parola 32 bit

void setup()
{
  lcd.begin(16, 2);
  lcd.setCursor(0,0);
  lcd.print("Square Generator");
  
  pinMode(3, OUTPUT);      // pin 3 = PWM out frequency: NOTA cambiare la corrispondente porta OCR2B e i bit COM2B0 e COM2B1 del timer2
//pinMode(11, OUTPUT);     // pin 11= PWM out frequency: NOTA cambiare la corrispondente porta OCR2A e i bit COM2A0 e COM2A1 del timer2

Setup_timer2(); 

  // disabilita gli interrupt per evitare variazioni nel timing
  bitClear(TIMSK0,TOIE0);              // disabilita il Timer0 (quello della funzione delay())
  bitSet(TIMSK2,TOIE2);              // abilita il Timer2 Interrupt

//frequenza iniziale di uscita = 50 Hz (impostare qualsiasi valore da 0 a 1023 Hz)
//freqOut=50;                        // impostare manualmente la frequenza da 0 a 1023 Hz

SquareWave();
}

void loop()
{
freqOut=analogRead(5);                 // decommentare per leggere il valore da 0 a 1023 dal potenziometro opzionale collegato alla porta analogica A5
Freqword=pow(2,32)*freqOut/REFCLOCK;  // calcolo frequenza di uscita rispetto al clock di riferimento
    if (counter_ms > 10) {         // timer di un secondo per evitare lo sfarfallio del display (ATTENZIONE delay() è disabilitato!)
        counter_ms=0;
  static unsigned char flag = 0;

flag ^= 1;

OCR2B=flag;
    // visualizza la frequenza sul display a intervalli di un secondo
    lcd.setCursor(0,1);
    lcd.print("Freq.  ");
    lcd.print(freqOut);      
    }
 }



//******************************************************************
// imposta il timer2
// impostare il prescaler a 1, PWM mode per la correzione di fase PWM, ovvero 16.000.000/512 = 31.372 Hz di clock
void Setup_timer2() {
// Timer2 Clock Prescaler a 1
  bitSet(TCCR2B, CS20);
  bitClear(TCCR2B, CS21);
  bitClear(TCCR2B, CS22);

     
// Timer2 PWM Mode impostato su Phase Correct PWM sul pin 3
  bitClear(TCCR2A, COM2B0);
  bitSet(TCCR2A, COM2B1);

// Timer2 PWM Mode impostato su Phase Correct PWM sul pin 11
  //bitClear (TCCR2A, COM2A0);
  //bitSet (TCCR2A, COM2A1);

// Mode 1 Phase Correct PWM
  bitSet(TCCR2A, WGM20); 
  bitClear(TCCR2A, WGM21);
  bitClear(TCCR2B, WGM22);
}


//******************************************************************
// Timer2 Interrupt a 31372,550 KHz = 32uSec (REFCLOCK per il generatore) runtime di 8 microsecondi
ISR(TIMER2_OVF_vect) 
{
static unsigned int PhaseOut;

  Phase=Phase+Freqword; // accumula fase accurata a 32 bit
  IntCNT=Phase >> 24;   // legge gli 8 bit superiori

PhaseOut=PhaseOut+freqOut;

OCR2B=RAMTable[PhaseOut>>8];   // pin 3
//OCR2A=RAMTable[PhaseOut>>8];   // pin 11

   if(IntCNT1++ == 125) {  
    counter_ms++; // incremento di 4 millisecondi
    IntCNT1=0;
   }   
}


void SquareWave()
{
  int t;
  for (int i = 0; i < 256; ++i) 
  {
if (i == 0) t = 0;
if (i == 64) t = 255;
if (i == 128) t = 0;
if (i == 192) t = 255;
    RAMTable[i] = t; // // inserisce i valori nella tabella
  }
}
